@@ -53,7 +53,7 @@
 #endif
 #endif /* atarist */
 #else
-#ifdef sparc
+#if defined(sparc) && !defined(__GNUC__)
 #include <alloca.h>
 #else
 char *alloca ();
@@ -81,7 +81,7 @@ static void insert_jump_n P((int, char *, char *, char *, unsigned));
 static void insert_op_2 P((int, char *, char *, int, int ));
 static int memcmp_translate P((unsigned char *, unsigned char *,
 			       int, unsigned char *));
-
+long re_set_syntax P((long));
 
 /* Define the syntax stuff, so we can do the \<, \>, etc.  */
 
@@ -281,11 +281,11 @@ enum regexpcode
    The argument SYNTAX is a bit-mask comprised of the various bits
    defined in regex.h.  */
 
-int
+long
 re_set_syntax (syntax)
-  int syntax;
+  long syntax;
 {
-  int ret;
+  long ret;
 
   ret = obscure_syntax;
   obscure_syntax = syntax;
@@ -293,7 +293,7 @@ re_set_syntax (syntax)
 }
 
 /* Set by re_set_syntax to the current regexp syntax to recognize.  */
-int obscure_syntax = 0;
+long obscure_syntax = 0;
 
 
 
@@ -377,8 +377,8 @@ int obscure_syntax = 0;
   }
 
 /* Subroutines for re_compile_pattern.  */
-static void store_jump (), insert_jump (), store_jump_n (),
-	    insert_jump_n (), insert_op_2 ();
+/* static void store_jump (), insert_jump (), store_jump_n (),
+	    insert_jump_n (), insert_op_2 (); */
 
 
 /* re_compile_pattern takes a regular-expression string
@@ -406,7 +406,7 @@ re_compile_pattern (pattern, size, bufp)
   register char *p = pattern;
   char *pend = pattern + size;
   register unsigned c, c1;
-  char *p1;
+  char *p0;
   unsigned char *translate = (unsigned char *) bufp->translate;
 
   /* Address of the count-byte of the most recently inserted `exactn'
@@ -599,7 +599,7 @@ re_compile_pattern (pattern, size, bufp)
 	      else if ((obscure_syntax & RE_BK_PLUS_QM)
 		       && c == '\\')
 		{
-		  int c1;
+		  /* int c1; */
 		  PATFETCH (c1);
 		  if (!(c1 == '+' || c1 == '?'))
 		    {
@@ -671,7 +671,7 @@ re_compile_pattern (pattern, size, bufp)
             }
 	  else
 	    BUFPUSH (charset);
-	  p1 = p;
+	  p0 = p;
 
 	  BUFPUSH ((1 << BYTEWIDTH) / BYTEWIDTH);
 	  /* Clear the whole map */
@@ -697,7 +697,7 @@ re_compile_pattern (pattern, size, bufp)
 	        }
               if (c == ']')
                 {
-                  if (p == p1 + 1)
+                  if (p == p0 + 1)
                     {
 		      /* If this is an empty bracket expression.  */
                       if ((obscure_syntax & RE_NO_EMPTY_BRACKETS) 
@@ -1229,13 +1229,9 @@ re_compile_pattern (pattern, size, bufp)
 static void
 store_jump (from, opcode, to)
      char *from, *to;
-#ifndef MSDOS
-     char opcode;
-#else
      int opcode;
-#endif /* MSDOS */
 {
-  from[0] = opcode;
+  from[0] = (char)opcode;
   STORE_NUMBER(from + 1, to - (from + 3));
 }
 
@@ -1248,11 +1244,7 @@ store_jump (from, opcode, to)
 
 static void
 insert_jump (op, from, to, current_end)
-#ifndef MSDOS
-     char op;
-#else
      int op;
-#endif /* MSDOS */
      char *from, *to, *current_end;
 {
   register char *pfrom = current_end;		/* Copy from here...  */
@@ -1275,14 +1267,10 @@ insert_jump (op, from, to, current_end)
 static void
 store_jump_n (from, opcode, to, n)
      char *from, *to;
-#ifndef MSDOS
-     char opcode;
-#else
      int opcode;
-#endif /* MSDOS */
      unsigned n;
 {
-  from[0] = opcode;
+  from[0] = (char)opcode;
   STORE_NUMBER (from + 1, to - (from + 3));
   STORE_NUMBER (from + 3, n);
 }
@@ -1298,11 +1286,7 @@ store_jump_n (from, opcode, to, n)
 
 static void
 insert_jump_n (op, from, to, current_end, n)
-#ifndef MSDOS
-     char op;
-#else
      int op;
-#endif /* MSDOS */
      char *from, *to, *current_end;
      unsigned n;
 {
@@ -1323,11 +1307,7 @@ insert_jump_n (op, from, to, current_end, n)
 
 static void
 insert_op_2 (op, there, current_end, num_1, num_2)
-#ifndef MSDOS
-     char op;
-#else
      int op;
-#endif /* MSDOS */
      char *there, *current_end;
      int num_1, num_2;
 {
@@ -1337,7 +1317,7 @@ insert_op_2 (op, there, current_end, num_1, num_2)
   while (pfrom != there)			       
     *--pto = *--pfrom;
   
-  there[0] = op;
+  there[0] = (char)op;
   STORE_NUMBER (there + 1, num_1);
   STORE_NUMBER (there + 3, num_2);
 }
@@ -1521,6 +1501,11 @@ re_compile_fastmap (bufp)
 	    if (SYNTAX (j) != (enum syntaxcode) k)
 	      fastmap[j] = 1;
 	  break;
+
+#else /* not emacs */
+	case syntaxspec:
+	case notsyntaxspec:
+	  break;
 #endif /* not emacs */
 
 	case charset:
@@ -1551,6 +1536,9 @@ re_compile_fastmap (bufp)
 		  fastmap[j] = 1;
 	      }
 	  break;
+
+	case unused:	/* pacify gcc -Wall */
+	  break;
 	}
 
       /* Get here means we have successfully found the possible starting
@@ -1739,7 +1727,7 @@ re_match (pbufp, string, size, pos, regs)
 int re_max_failures = 2000;
 
 /* Routine used by re_match_2.  */
-static int memcmp_translate ();
+/* static int memcmp_translate (); *//* already declared */
 
 
 /* Structure and accessing macros used in re_match_2:  */
@@ -1778,12 +1766,12 @@ struct register_info
 
 #define PUSH_FAILURE_POINT(pattern_place, string_place)			\
   {									\
-    short last_used_reg, this_reg;					\
+    long last_used_reg, this_reg;					\
 									\
     /* Find out how many registers are active or have been matched.	\
        (Aside from register zero, which is only set at the end.)  */	\
     for (last_used_reg = RE_NREGS - 1; last_used_reg > 0; last_used_reg--)\
-      if (regstart[last_used_reg] != (unsigned char *) -1)		\
+      if (regstart[last_used_reg] != (unsigned char *)(-1L))		\
         break;								\
 									\
     if (stacke - stackp < NUM_FAILURE_ITEMS)				\
@@ -2003,7 +1991,7 @@ re_match_2 (pbufp, string1_arg, size1, string2_arg, size2, pos, regs, mstop)
      failed.  */
   for (mcnt = 0; mcnt < RE_NREGS; mcnt++)
     {
-      regstart[mcnt] = regend[mcnt] = (unsigned char *) -1;
+      regstart[mcnt] = regend[mcnt] = (unsigned char *) (-1L);
       IS_ACTIVE (reg_info[mcnt]) = 0;
       MATCHED_SOMETHING (reg_info[mcnt]) = 0;
     }
@@ -2118,7 +2106,7 @@ re_match_2 (pbufp, string1_arg, size1, string2_arg, size2, pos, regs, mstop)
 		regs->end[0] = d - string2 + size1;
 	      for (mcnt = 1; mcnt < RE_NREGS; mcnt++)
 		{
-		  if (regend[mcnt] == (unsigned char *) -1)
+		  if (regend[mcnt] == (unsigned char *)(-1L))
 		    {
 		      regs->start[mcnt] = -1;
 		      regs->end[mcnt] = -1;
@@ -2536,6 +2524,13 @@ re_match_2 (pbufp, string1_arg, size1, string2_arg, size2, pos, regs, mstop)
           SET_REGS_MATCHED;
 	  break;
 
+	case before_dot:
+	case at_dot:
+	case after_dot:
+	case syntaxspec:
+	case notsyntaxspec:
+	  break;
+
 #endif /* not emacs */
 
 	case begbuf:
@@ -2597,13 +2592,13 @@ re_match_2 (pbufp, string1_arg, size1, string2_arg, size2, pos, regs, mstop)
           if (d >= string1 && d <= end1)
 	    dend = end_match_1;
           /* Restore register info.  */
-          last_used_reg = (short) *--stackp;
+          last_used_reg = (long) *--stackp;
           
           /* Make the ones that weren't saved -1 or 0 again.  */
           for (this_reg = RE_NREGS - 1; this_reg > last_used_reg; this_reg--)
             {
-              regend[this_reg] = (unsigned char *) -1;
-              regstart[this_reg] = (unsigned char *) -1;
+              regend[this_reg] = (unsigned char *) (-1L);
+              regstart[this_reg] = (unsigned char *) (-1L);
               IS_ACTIVE (reg_info[this_reg]) = 0;
               MATCHED_SOMETHING (reg_info[this_reg]) = 0;
             }
@@ -2646,7 +2641,7 @@ memcmp_translate (s1, s2, len, translate)
 
 /* Entry points compatible with 4.2 BSD regex library.  */
 
-#ifndef emacs
+#if !defined(emacs) && !defined(GAWK)
 
 static struct re_pattern_buffer re_comp_buf;
 
@@ -2680,7 +2675,7 @@ re_exec (s)
   return 0 <= re_search (&re_comp_buf, s, len, 0, len,
 			 (struct re_registers *) 0);
 }
-#endif /* not emacs */
+#endif /* not emacs && not GAWK */
 
 
 
@@ -2771,7 +2766,7 @@ main (argc, argv)
 
   /* Allow a command argument to specify the style of syntax.  */
   if (argc > 1)
-    obscure_syntax = atoi (argv[1]);
+    obscure_syntax = atol (argv[1]);
 
   buf.allocated = 40;
   buf.buffer = (char *) malloc (buf.allocated);
